#region Copyright (c) 2002, newtelligence AG. All rights reserved.
// Copyright (c) 2002, newtelligence AG. (http://www.newtelligence.com)
// All rights reserved.
#endregion



using System;
using System.Reflection;
using System.Collections;
using System.Text.RegularExpressions;
using System.Runtime.Remoting.Messaging;

namespace newtelligence.Aspects.Constraints
{

	/// <summary>
	/// This is interface must be implemented by all constraint attributes that 
	/// are designed to work with the System.Web.Services.Security.CheckArgumentsAttribute
	/// context attribute for automatic evaluation of constraints on assignment.
	/// </summary>
	public interface IConstraintAttribute
	{  
		/// <summary>
		/// Tests the condition and returns false if the
		/// condition is not met.
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="Member">Member on which this actication is performed</param>
		/// <param name="parameter">Parameter being passed.</param>
		/// <returns>Indication of whether the validation succeeded or failed</returns>
		bool TestCondition(object Value, object Container, MemberInfo Member, ParameterInfo parameter);
		/// <summary>
		/// Tests the condition and throws a ConstraintViolationException
		/// if the condition is not met.
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="parameter">Parameter being passed</param>
		/// <param name="Member">Metadata of the argument type</param>
		void TestConditionAndThrow(object Value, object Container, MemberInfo Member, ParameterInfo parameter);
	}

	/// <summary>
	/// Values to be used with ComparisionConstraints.
	/// </summary>
	public enum CompareAgainst
	{
		/// <summary>
		/// This option indicates that the ComparisonConstraint shall 
		/// compare against a member or property. In this case, the value
		/// supplied to the constraint is the name of a member or property
		/// of the validated "container", which is the Web service 
		/// implementation class.
		/// </summary>
		FieldOrProperty,
		/// <summary>
		/// This option indicates that the Comparison constraint shall 
		/// compare against a value. In this case, the value supplied 
		/// to the constraint attribute is a constant value.
		/// </summary>
		Value
	}

	/// <summary>
	/// 
	/// </summary>
	public sealed class Constraints 
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="method"></param>
		/// <param name="parameters"></param>
		/// <param name="arguments"></param>
		public static void CheckArgumentsAndThrow(MethodInfo method, ParameterInfo[] parameters, object[] arguments )
		{
			int i=0;
			foreach ( ParameterInfo parameter in parameters )
			{
				if ( parameter.IsDefined(typeof(Constraint),true) )
				{
					Constraint[] constraints = (Constraint[])parameter.GetCustomAttributes(typeof(Constraint),true);
				
					foreach( Constraint constraint in constraints )
					{
						constraint.TestConditionAndThrow(arguments[i],null,method, parameter);
					}
				}
				i++;
			}
		}

		internal static void ValidateAndThrow(object ObjectToValidate )
		{
		
			MemberInfo[] members = ObjectToValidate.GetType().GetMembers(BindingFlags.Instance|BindingFlags.GetField|BindingFlags.GetProperty|BindingFlags.Public);
			foreach ( MemberInfo member in members )
			{
				if ( member.IsDefined(typeof(Constraint),true) )
				{
					Constraint[] constraints = (Constraint[])member.GetCustomAttributes(typeof(Constraint),true);
					PropertyInfo property = member as PropertyInfo;
					FieldInfo field = member as FieldInfo;

					foreach( Constraint constraint in constraints )
					{
						if ( field != null )
						{
							constraint.TestConditionAndThrow(field.GetValue(ObjectToValidate),ObjectToValidate,member,null);
						}
						else if ( property !=  null )
						{
							constraint.TestConditionAndThrow(property.GetValue(ObjectToValidate,null),ObjectToValidate,property,null);
						}
					}
				}
			}
		}

		internal static MemberInfo[] Validate(object ObjectToValidate )
		{
			ArrayList invalidMembers = new ArrayList();
			MemberInfo[] members = ObjectToValidate.GetType().GetMembers(BindingFlags.Instance|BindingFlags.GetField|BindingFlags.GetProperty|BindingFlags.Public);
			foreach ( MemberInfo member in members )
			{
				if ( member.IsDefined(typeof(Constraint),true) )
				{
					Constraint[] constraints = (Constraint[])member.GetCustomAttributes(typeof(Constraint),true);
					PropertyInfo property = member as PropertyInfo;
					FieldInfo field = member as FieldInfo;

					foreach( Constraint constraint in constraints )
					{
						if ( field != null )
						{
							if ( !constraint.TestCondition(field.GetValue(ObjectToValidate),ObjectToValidate, member,null) )
								invalidMembers.Add(field);

						}
						else if ( property !=  null )
						{
							if ( !constraint.TestCondition(property.GetValue(ObjectToValidate,null),ObjectToValidate, property,null))
								invalidMembers.Add(property);
						}
					}
				}
			}
			return (MemberInfo[])invalidMembers.ToArray(typeof(MemberInfo));
		}
	}

	/// <summary>
	/// Exception base class for all exceptions thrown by the constraint classes. 
	/// And exception is thrown whenever setting a property or member value violates
	/// a constraint.
	/// </summary>
	[Serializable]
	public class ConstraintViolationException : ArgumentOutOfRangeException 
	{
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		protected internal object _container;
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		protected internal MemberInfo _field;
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		protected internal Constraint _constraint;
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		protected internal ParameterInfo _parameter;
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="constraint">[To be supplied.]</param>
		/// <param name="value">[To be supplied.]</param>
		/// <param name="container">[To be supplied.]</param>
		/// <param name="member">[To be supplied.]</param>
		/// <param name="parameter">Parameter being passed.</param>
		/// 
		internal ConstraintViolationException(Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter)
			:base(parameter!=null?parameter.Name:"Field:"+member.Name,value,
			constraint.GetViolationMessage( constraint,value,container,member, parameter))
		{
			_container = Container;
			_constraint = constraint;
			_field = member;
			_parameter = parameter;
		}
		/// <summary>
		/// Gets the constraint definition that has been violated 
		/// </summary>
		public Constraint Constraint
		{
			get 
			{ 
				return _constraint;
			}
		}
		
		/// <summary>
		/// Gets the container of the value that violated the constraint
		/// </summary>
		public object Container
		{
			get
			{
				return _container;
			}
		}

		/// <summary>
		/// Gets the field information if the violating member was indeed a field; otherwise null.
		/// </summary>
		public MemberInfo Field
		{
			get
			{
				return _field;
			}
		}

		/// <summary>
		/// Gets the parameter information, if the violating member was a parameter; otherwise null.
		/// </summary>
		public ParameterInfo Parameter
		{
			get
			{
				return _parameter;
			}
		}

		
	}

	/// <summary>
	/// This is the base class for all constraints implemented in this library. 
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public abstract class Constraint : 
		Attribute, 
		IConstraintAttribute
	{
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		public Constraint()	
		{
		}

		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="Value">[To be supplied.]</param>
		/// <param name="Container">[To be supplied.]</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>[To be supplied.]</returns>
		public virtual bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			return false;
		}
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="Value">[To be supplied.]</param>
		/// <param name="Container">[To be supplied.]</param>
		/// <param name="parameter"></param>
		/// <param name="member">[To be supplied.]</param>
		public virtual void TestConditionAndThrow(object Value,object Container,MemberInfo member, ParameterInfo parameter)
		{
			if ( !TestCondition(Value,Container, member, parameter) )
			{
				throw( new ConstraintViolationException(this,Value,Container,member,parameter));
			}
		}

		/// <summary>
		/// Message explaining the constraint violation. Must be overridden.
		/// </summary>
		public abstract string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter );
		/// <summary>
		/// Gets the name of the violating expression
		/// </summary>
		/// <param name="constraint"></param>
		/// <param name="value"></param>
		/// <param name="container"></param>
		/// <param name="member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public virtual string GetViolatorName( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			if ( parameter != null )
			{
				return String.Format("parameter {0} (value: {1}) of member {2} in {3}",
					                 parameter.Name, value, member.Name, container.GetType().Name);
			}
			else if ( member != null )
			{
				return String.Format("member {0} (value: {1}) in {2}",
					member.Name, value, container.GetType().Name);
			}
			else
			{
				return "unknown member";
			}
		}

	}
	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is not null.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class NotNull : Constraint
	{
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		public NotNull()
		{
		}
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="Value">[To be supplied.]</param>
		/// <param name="Container">[To be supplied.]</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>[To be supplied.]</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			return (Value != null && Value.GetType() != typeof(DBNull));
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{0} must not be NULL", GetViolatorName( constraint, value, container, member, parameter ));

		}
	}
	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and can be used to flag a method parameter declaration nullable. This attribute will 
	/// always validate positively.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class Nullable : Constraint
	{
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		public Nullable()
		{
		}
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="Value">[To be supplied.]</param>
		/// <param name="Container">[To be supplied.]</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>[To be supplied.]</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			return true;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{0} must not be NULL",GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	
	/// <summary>
	/// [To be supplied.]
	/// </summary>
	public abstract class ComparisonConstraint : Constraint
	{
		/// <summary>
		/// [To be supplied.]
		/// </summary>
		internal object compareValue;

		/// <summary>
		/// [To be supplied.]
		/// </summary>
		internal CompareAgainst against;

		/// <summary>
		/// [To be supplied.]
		/// </summary>
		/// <param name="against">[To be supplied.]</param>
		/// <param name="compareValue">[To be supplied.]</param>
		public ComparisonConstraint(CompareAgainst against, object compareValue)
		{
			this.against = against;
			this.compareValue=compareValue;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public ComparisonConstraint(object compareValue)
		{
			this.against = CompareAgainst.Value;
			this.compareValue=compareValue;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="value"></param>
		/// <param name="container"></param>
		/// <param name="member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object value,object container, MemberInfo member, ParameterInfo parameter)
		{
			IComparable compare = (IComparable)value;
			object cmpval = GetCompareValue( this,value, container, member, parameter);		
			
			if ( cmpval == null )
			{
				return false;
			}
			return (TestCompareResult(compare.CompareTo(Convert.ChangeType(cmpval,compareValue.GetType()))));
		}

		private object GetCompareValue(Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter)
		{
			
			object cmpval = null;
			if ( against == CompareAgainst.Value )
			{
				cmpval = compareValue;					
			}
			else
			{
				MemberInfo mi = container.GetType().GetMember((string)compareValue)[0];
				if ( mi is PropertyInfo )
				{
					cmpval =((PropertyInfo)mi).GetValue(container,new object[0]);
				}
				else if ( mi is FieldInfo )
				{
					cmpval =((FieldInfo)mi).GetValue(container);
				}
			}
			return cmpval;
			
		}

		private string GetCompareMemberName(Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter)
		{
			
				if ( against == CompareAgainst.FieldOrProperty )
				{
					MemberInfo mi = container.GetType().GetMember((string)compareValue)[0];
					return mi.Name;
				}
				else
				{
					return "$Parameter";
				}
			
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="member">[To be supplied.]</param>
		/// <param name="parameter"></param>
		public override void TestConditionAndThrow(object Value,object Container,MemberInfo member, ParameterInfo parameter)
		{
			if ( !TestCondition(Value,Container,member,parameter) )
			{
				throw( new ConstraintViolationException(this,Value,Container,member,parameter));
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public abstract bool TestCompareResult(int result);
		

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public virtual string ConstraintName
		{
			get
			{
				return GetType().Name;
			}
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			if ( against == CompareAgainst.Value )
			{
				return String.Format("{2} must be {0} {1}", ConstraintName, GetCompareValue( constraint,value,container,member, parameter), GetViolatorName( constraint, value, container, member, parameter ) );
			}
			else
			{
				return String.Format("{3} must be {0} member {1}. Member value is {2}", ConstraintName, GetCompareMemberName( constraint,value,container,member, parameter), GetCompareValue( constraint,value,container,member, parameter), GetViolatorName( constraint, value, container, member, parameter ) );
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is less than a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class LessThan : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public LessThan(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public LessThan(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result < 0);
		}


		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "less than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is equal to a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class EqualTo : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public EqualTo(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public EqualTo(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result == 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "equal to";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is not equal to a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class NotEqualTo : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public NotEqualTo(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public NotEqualTo(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result != 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "not equal to";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is greater than a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class GreaterThan : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public GreaterThan(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public GreaterThan(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result > 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "greater than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is greater to equal to a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class GreaterEqualTo : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public GreaterEqualTo(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public GreaterEqualTo(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result >= 0);
		}

		/// <summary>
		/// Human readble name of this constraint.
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "greater than or equal to";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is less or equal to a 
	/// constant value that is passed as the attribute argument.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class LessEqualTo : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="against"></param>
		/// <param name="compareValue"></param>
		public LessEqualTo(CompareAgainst against, object compareValue)
			:base(against,compareValue)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="compareValue"></param>
		public LessEqualTo(object compareValue):base(compareValue)
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result <= 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "less than or equal to";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is earlier than today.
	/// (This does, of course, assume that the method parameter declaration is of type System.DateTime)
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class EarlierThanToday : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		public EarlierThanToday()
			:base(DateTime.Today)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result < 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "earlier than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is earlier than now
	/// (This does, of course, assume that the method parameter declaration is of type System.DateTime)
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class EarlierThanNow : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		public EarlierThanNow()
			:base(DateTime.Now)
		{
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result < 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "earlier than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is later than today.
	/// (This does, of course, assume that the method parameter declaration is of type System.DateTime)
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class LaterThanToday : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		public LaterThanToday()
			:base(DateTime.Today.AddSeconds(24*60*60-1))
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result > 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "later than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is later than "now".
	/// (This does, of course, assume that the method parameter declaration is of type System.DateTime)
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class LaterThanNow : ComparisonConstraint
	{
		/// <summary>
		/// 
		/// </summary>
		public LaterThanNow()
			:base(DateTime.Now)
		{
			
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="result"></param>
		/// <returns></returns>
		public override bool TestCompareResult(int result)
		{
			return (result > 0);
		}

		/// <summary>
		/// Human readable name of the constraint
		/// </summary>
		public override string ConstraintName
		{
			get
			{
				return "later than";
			}
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is not empty.
	/// This attribute can be used with all data types that have public properties named
	/// "Count" or "Length", which are convertible to Int32.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class NotEmpty : Constraint
	{
		/// <summary>
		/// 
		/// </summary>
		public NotEmpty()
		{
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			Type t = Value.GetType();
			MemberInfo[] mi = t.GetMember("Count");
			if ( mi == null || mi.Length==0)
			{
				mi = t.GetMember("Length");
			}
			
			if ( mi != null && mi.Length>0)
			{
				PropertyInfo pi = mi[0] as PropertyInfo;
				FieldInfo fi = mi[0] as FieldInfo;

				if (pi != null)
				{
					return Convert.ToInt32(pi.GetValue(Value,new object[0])) > 0;
				}
				else if (fi != null)
				{
					return Convert.ToInt32(fi.GetValue(Value)) > 0;
				}
			}
			return false;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{0} must not be empty or null.",GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument has at least the
	/// number of elements specified in the attribute.
	/// This attribute can be used with all data types that have public properties named
	/// "Count" or "Length", which are convertible to Int32.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class MinElements : Constraint
	{
		/// <summary>
		/// 
		/// </summary>
		private int n;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="n">Minimum number of elements the argument must contain.</param>
		public MinElements(int n)
		{
			this.n = n;
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			ICollection c = (ICollection)Value;
			return c.Count>=n;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{1} must have at least {0} elements.",n, GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument has at most the
	/// number of elements specified in the attribute.
	/// This attribute can be used with all data types that have public properties named
	/// "Count" or "Length", which are convertible to Int32.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class MaxElements : Constraint
	{
		/// <summary>
		/// 
		/// </summary>
		private int n;
		/// <summary>
		/// 
		/// </summary>
		/// <param name="n">Maxmimum number of elements the argument may contain.</param>
		public MaxElements(int n)
		{
			this.n = n;
		}
		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			ICollection c = (ICollection)Value;
			return c.Count<=n;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{1} must have at most {0} elements.",n,GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the string value passed as the respective method argument has a 
	/// minimum length as specified in the attribute.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class MinLength : Constraint
	{
		private int n;

		/// <summary>
		/// This is a constraint attribute that can be applied to a method parameter declaration.
		/// The attribute works in conjunction with the CheckArgumentsAttribute
		/// and ensures that the string value passed as the respective method argument has a 
		/// minimum length as specified in the attribute.
		/// </summary>
		/// <param name="n">Minimum length of the string expression</param>
		public MinLength(int n)
		{
			this.n = n;
		}


		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			string s = Value as string;
			if ( s != null )
                return s.Length>=n;
			else
				return false;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{1} must have at least {0} characters.",n,GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the string value passed as the respective method argument has a 
	/// maximum length as specified in the attribute.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public sealed class MaxLength : Constraint
	{
		private int n;

		/// <summary>
		/// This is a constraint attribute that can be applied to a method parameter declaration.
		/// The attribute works in conjunction with the CheckArgumentsAttribute
		/// and ensures that the string value passed as the respective method argument has a 
		/// maximum length as specified in the attribute.
		/// </summary>
		/// <param name="n">Maximum length of the string expression</param>
		public MaxLength(int n)
		{
			this.n = n;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			string s = Value as string;
			if ( s != null )
				return s.Length<=n;
			else 
				return false;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{1} must have at most {0} characters.",n,GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is 
	/// between the lower and upper boundaries specified in the attribute.
	/// This attribute can be used with all data types that implement the IComparable
	/// interface and whose data type is conversion comptible with the supplied 
	/// boundary constants.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public class Between : Constraint
	{
		private object min;
		private object max;

		/// <summary>
		/// This is a constraint attribute that can be applied to a method parameter declaration.
		/// The attribute works in conjunction with the CheckArgumentsAttribute
		/// and ensures that the value passed as the respective method argument is 
		/// between the lower and upper boundaries specified in the attribute.
		/// This attribute can be used with all data types that implement the IComparable
		/// interface and whose data type is conversion comptible with the supplied 
		/// boundary constants.
		/// </summary>
		/// <param name="min">Lower boundary value to compare against</param>
		/// <param name="max">Upper boundary value to compare against</param>
		public Between(object min, object max)
		{
			this.min = min;
			this.max = max;
		}

		/// <summary>
		/// Tests the condition and returns false if the
		/// condition is not met. <see cref="IConstraintAttribute"/>
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>Indication of whether the validation succeeded or failed</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			IComparable compare = (IComparable)Value;
			return (compare.CompareTo(Convert.ChangeType(min,Value.GetType())) >= 0 && compare.CompareTo(Convert.ChangeType(max,Value.GetType())) <= 0);
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("Value of {2} must be between {0} and {1} (inclusive).",min, max,GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is 
	/// between the lower and upper boundaries specified in the attribute, with the
	/// boundary values excluded.
	/// This attribute can be used with all data types that implement the IComparable
	/// interface and whose data type is conversion comptible with the supplied 
	/// boundary constants.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public class BetweenExclusive : Constraint
	{
		object min;
		object max;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public BetweenExclusive(object min, object max)
		{
			this.min = min;
			this.max = max;
		}

		/// <summary>
		/// Tests the condition and returns false if the
		/// condition is not met. <see cref="IConstraintAttribute"/>
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>Indication of whether the validation succeeded or failed</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			IComparable compare = (IComparable)Value;
			return (compare.CompareTo(min) > 0 && compare.CompareTo(max) < 0);
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("Value of {2} must be between {0} and {1} (exclusive).",min, max, GetViolatorName( constraint, value, container, member, parameter ));

		}
	}
	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is 
	/// not between the lower and upper boundaries specified in the attribute.
	/// This attribute can be used with all data types that implement the IComparable
	/// interface and whose data type is conversion comptible with the supplied 
	/// boundary constants.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public class NotBetween : Constraint
	{
		object min;
		object max;

		/// <summary>
		/// This is a constraint attribute that can be applied to a method parameter declaration.
		/// The attribute works in conjunction with the CheckArgumentsAttribute
		/// and ensures that the value passed as the respective method argument is 
		/// not between the lower and upper boundaries specified in the attribute.
		/// This attribute can be used with all data types that implement the IComparable
		/// interface and whose data type is conversion comptible with the supplied 
		/// boundary constants. 
		/// </summary>
		/// <param name="min">Lower boundary of the exclusion set</param>
		/// <param name="max">Upper boundary of the exclusion set</param>
		public NotBetween(object min, object max)
		{
			this.min = min;
			this.max = max;
		}

		/// <summary>
		/// Tests the condition and returns false if the
		/// condition is not met. <see cref="IConstraintAttribute"/>
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>Indication of whether the validation succeeded or failed</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			IComparable compare = (IComparable)Value;
			return (compare.CompareTo(min) < 0 || compare.CompareTo(max) > 0);
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("Value of {2} must not be between {0} and {1}.",min, max, GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the string value passed as the respective method argument is 
	/// matches the regular expression specified in the attribute.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public class Match : Constraint
	{
		string pattern;
		
		/// <summary>
		/// This is a constraint attribute that can be applied to a method parameter declaration.
		/// The attribute works in conjunction with the CheckArgumentsAttribute
		/// and ensures that the string value passed as the respective method argument is 
		/// matches the regular expression specified in the attribute.
		/// </summary>
		/// <param name="pattern">Regular expression pattern as defined in System.Text.RegularExpressions.RegEx.</param>
		public Match(string pattern)
		{
			this.pattern = pattern;
		}

		/// <summary>
		/// Tests the condition and returns false if the
		/// condition is not met. <see cref="IConstraintAttribute"/>
		/// </summary>
		/// <param name="Value">Value object to be tested</param>
		/// <param name="Container">Container object of the value</param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns>Indication of whether the validation succeeded or failed</returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			string s = Value as string;

			if ( s == null )
				return false;

			Regex rex = new Regex(pattern);
			MatchCollection mc;

			mc = rex.Matches(s);
			foreach( System.Text.RegularExpressions.Match m in mc )
			{
				if ( m.Value == s )
				{
					return true;					
				}
			}
			return false;
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			return String.Format("{1} must match the regular expression {0}", pattern, GetViolatorName( constraint, value, container, member, parameter ));

		}
	}

	/// <summary>
	/// This is the interface to be implemented by simple range providers compatible 
	/// with the OneOfAttribute. A simple range provider returns a set of values
	/// indexed by a name, which is tested against by the OneOfAttribute. 
	/// </summary>
	interface IRangeProvider
	{
		/// <summary>
		/// 
		/// </summary>
		object[] GetRange(string name);
	}

	/// <summary>
	/// This is the interface to be implemented by range constraint providers compatible 
	/// with the OneOfAttribute. A range constraint provider performs a test of a value
	/// against an internally known set of values indexed by a name.
	/// </summary>
	interface IRangeConstraintProvider
	{
		/// <summary>
		/// This method tests a value against an internally known set of
		/// values and returns false if the value is not contained in the set 
		/// or otherwise doesn't map to the set.
		/// </summary>
		/// <param name="name">Name of the set to test against. May be null
		/// or empty for the default set, if such a set exists.</param>
		/// <param name="value">Value to be tested.</param>
		/// <param name="Container">Calling object.</param>
		/// <returns></returns>
		bool TestRange(string name, object value, object Container);
	}

	/// <summary>
	/// This is a simple implementation of a range provider returning an empty set. 
	/// </summary>
	public class DefaultRangeProvider : IRangeProvider
	{
		/// <summary>
		/// 
		/// </summary>
		/// <param name="name"></param>
		/// <returns></returns>
		public object[] GetRange(string name)
		{
			return new object[0];
		}
	}

	/// <summary>
	/// This is a constraint attribute that can be applied to a method parameter declaration.
	/// The attribute works in conjunction with the CheckArgumentsAttribute
	/// and ensures that the value passed as the respective method argument is 
	/// contained in a set of values that is either provides literally as arguments to
	/// the attribute or indirectly through a range provider. Using range providers allows
	/// dynamic checks.
	/// </summary>
	[AttributeUsage(AttributeTargets.Parameter|AttributeTargets.Property|AttributeTargets.Field|AttributeTargets.ReturnValue)]
	public class OneOf : Constraint
	{
		/// <summary>
		/// 
		/// </summary>
		private ArrayList values=new ArrayList();
		/// <summary>
		/// 
		/// </summary>
		private Type rangeProvider=null;
		/// <summary>
		/// 
		/// </summary>
		private string rangeName=null;

		/// <summary>
		/// 
		/// </summary>
		/// <param name="rangeProvider">Specifies the type of the range provider to use.</param>
		/// <param name="rangeName">Name of the range to query for.</param>
		public OneOf( Type rangeProvider, string rangeName)
		{
			this.rangeProvider=rangeProvider;
			this.rangeName=rangeName;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="v"></param>
		public OneOf( params object[] v)
		{
			values.AddRange(v);
		}
		
		/// <summary>
		/// 
		/// </summary>
		/// <param name="Value"></param>
		/// <param name="Container"></param>
		/// <param name="Member"></param>
		/// <param name="parameter"></param>
		/// <returns></returns>
		public override bool TestCondition(object Value,object Container, MemberInfo Member, ParameterInfo parameter)
		{
			if ( rangeProvider != null )
			{
				object op = Activator.CreateInstance(rangeProvider);
				if ( op is IRangeConstraintProvider )
				{
					IRangeConstraintProvider rp = Activator.CreateInstance(rangeProvider) as IRangeConstraintProvider;
					return rp.TestRange(rangeName,Value,Container);
				}
				else if ( op is IRangeProvider )
				{
					IRangeProvider rp = Activator.CreateInstance(rangeProvider) as IRangeProvider;
					values.Clear();
					values.AddRange( rp.GetRange(rangeName) );
				}
				else
				{
					throw( new InvalidCastException());
				}
			}
			return values.Contains(Value);
		}

		/// <summary>
		/// Message explaining the constraint violation.
		/// </summary>
		public override string GetViolationMessage( Constraint constraint, object value, object container, MemberInfo member, ParameterInfo parameter )
		{
			System.Collections.ArrayList sc = new System.Collections.ArrayList();
			foreach( object o in values )
			{
				sc.Add( o.ToString());
			}
			string[] sarr = (string[])sc.ToArray(typeof(string));
			string sval = String.Join(",",sarr);
			return String.Format("{1} must be one of the following values: {0}",sval,GetViolatorName( constraint, value, container, member, parameter ));
		}
	}

	/// <summary>
	/// This class validates all field and property values in an object graph 
	/// based on their constraint attributes.
	/// </summary>
	public class ConstraintValidator
	{
		private ArrayList seenObjects = new ArrayList();

		/// <summary>
		/// Validates an object graph
		/// </summary>
		/// <param name="obj">Root object</param>
		public void Validate( object obj )
		{
			Validate( obj, true );

		}

		/// <summary>
		/// Walks the object graph and validates all fields and properties 
		/// encountered in objects contained in the graph.
		/// </summary>
		/// <param name="obj">Root object</param>
		/// <param name="deep">Indicates whether the function shall walk children</param>
		public void Validate( object obj, bool deep )
		{
			seenObjects.Clear();
			InternalValidate( obj, deep );
		}

		/// <summary>
		/// Walks the object graph and validates all fields and properties 
		/// encountered in objects contained in the graph.
		/// </summary>
		/// <param name="obj">Root object</param>
		/// <param name="deep">Indicates whether the function shall walk children</param>
		public void InternalValidate( object obj, bool deep )
		{
			Type objectType = obj.GetType();

			foreach( FieldInfo fi in objectType.GetFields(BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public) )
			{
				object objectValue = fi.GetValue( obj );

				if ( seenObjects.Contains( objectValue ) )
					continue;

				if ( fi.IsDefined(typeof(IConstraintAttribute),true))
				{
					foreach(IConstraintAttribute constraint in fi.GetCustomAttributes(typeof(IConstraintAttribute),true))
					{
						constraint.TestConditionAndThrow( objectValue, obj, fi, null );
					}
				}

				if ( deep && objectValue != null && !fi.FieldType.IsPrimitive )
				{
					//seenObjects.Add( objectValue );
					InternalValidate( objectValue, deep );
				}
				
			}

			foreach( PropertyInfo pi in objectType.GetProperties(BindingFlags.Instance|BindingFlags.NonPublic|BindingFlags.Public) )
			{						   
				ParameterInfo[] paramInfo = pi.GetIndexParameters();
				if (paramInfo != null && paramInfo.Length > 0 )
					continue;

				object objectValue = pi.GetValue( obj,null );

				if ( seenObjects.Contains( objectValue ) )
					continue;

				if ( pi.IsDefined(typeof(IConstraintAttribute),true))
				{
					foreach(IConstraintAttribute constraint in pi.GetCustomAttributes(typeof(IConstraintAttribute),true))
					{
						constraint.TestConditionAndThrow( objectValue, obj, pi, null );
					}
				}

				if ( deep && objectValue != null && !pi.PropertyType.IsPrimitive )
				{
					seenObjects.Add( objectValue );
					InternalValidate( objectValue, deep );
				}
			}
		}
	}
}
