using System;
using System.IO;
using System.Reflection;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Xml.Schema;

namespace newtelligence.Samples.Metadata
{
	/// <summary>
	/// This class validates object graphs and XML streams using
	/// XML Schema type definitions.
	/// </summary>
	public class SchemaValidator
	{
		private ArrayList validationSchemas;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="schemaStream">Stream containing an XML Schema document</param>
		/// <remarks>
		/// Use this constructor variant when you have an XML Schema document
		/// stored as an embedded stream inside your assembly. 
		/// Assembly.GetExecutingAssembly().GetManifextResourceStream("...");
		/// </remarks>
		public SchemaValidator(System.IO.Stream schemaStream)
		{
			validationSchemas = new ArrayList();
			XmlTextReader xmlReader = new XmlTextReader( schemaStream );
			validationSchemas.Add( XmlSchema.Read( xmlReader, new ValidationEventHandler( this.OnValidationEvent ) ) );
		}

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="schema">XML schema object</param>
		public SchemaValidator(XmlSchema schema)
		{
			validationSchemas = new ArrayList();
			validationSchemas.Add( schema );
		}

		/// <summary>
		/// Allows adding another schema object instance to be added
		/// </summary>
		/// <param name="schema">XML schema object</param>
		public void AddSchema( XmlSchema schema )
		{
			validationSchemas.Add( schema );
		}

		/// <summary>
		/// This method is called when validation errors occur and throws
		/// an exception.
		/// </summary>
		/// <param name="sender">Sender</param>
		/// <param name="e">Event arguments</param>
		private void OnValidationEvent(object sender,ValidationEventArgs e	)
		{
			throw e.Exception;
		}

		/// <summary>
		/// This method allows validating an XML instance made available
		/// through an XML reader.
		/// </summary>
		/// <param name="xmlStream">Input XML stream</param>
		/// <returns>Validated XML document instance</returns>
		public XmlDocument ValidateXmlStream( XmlReader xmlStream )
		{
			XmlValidatingReader xvr = new XmlValidatingReader( xmlStream );
			foreach( XmlSchema schema in validationSchemas ) 
			{
				xvr.Schemas.Add( schema );
			}
			xvr.ValidationEventHandler += new ValidationEventHandler(this.OnValidationEvent);
			xvr.ValidationType = ValidationType.Schema;
			// just read to end.
			XmlDocument retVal = new XmlDocument();
			retVal.Load( xvr );
			return retVal;
		}

		/// <summary>
		/// This method allows validating an object graph 
		/// </summary>
		/// <param name="obj">Root object</param>
		/// <returns>Returns the serialized object graph.</returns>
		public string ValidateObjectGraph( object obj )
		{
			string serializedResult;
			StringWriter sw = new StringWriter();
			XmlSerializer xs = new XmlSerializer( obj.GetType());
			xs.Serialize(sw,obj);
			serializedResult = sw.GetStringBuilder().ToString();
			ValidateXmlStream( new XmlTextReader( new StringReader( serializedResult ) ) );
			return serializedResult;
		}
	}
}
